

#include "Grid_M.hpp"

#include <qmath.h>
#include <iostream>


QGrid::QGrid(QQuickItem * parent)
    : QQuickItem(parent), m_start_time(-1.0) {

    setFlag(QQuickItem::ItemHasContents);

    m_timer_id = startTimer(100);
    m_start_time = -1.0;
}


QGrid::~QGrid() {
    killTimer(m_timer_id);
    ClearData();
}

void QGrid::timerEvent(QTimerEvent *event) {
    if (event->timerId() == m_timer_id) {
        update();
    }
}


void QGrid::ClearData() {

    if(!m_list_vertics.empty()) {

        m_list_vertics.erase(m_list_vertics.begin(), m_list_vertics.end());
        m_list_vertics.clear();
    }
}

bool QGrid::makeGrid(unsigned int totalWidthDivsion, unsigned int totalHeightDivision,
                                                            qreal wndWidth, qreal wndHeight)
{
    ClearData(); //clear all the data

    struct Point
    {
        qreal x;
        qreal y;
    };
    const Point vertexOffsets[] = {
        { 0.0, 0.0 },
        { 1.0, 0.0 },
        { 0.0, 1.0 },
        { 1.0, 0.0 },
        { 1.0, 1.0 },
        { 0.0, 1.0 }
    };

    const Point textureCoords[] = {
        { 0.0, 0.0 },
        { 1.0, 1.0 },
        { 0.0, 1.0 },
        { 0.0, 0.0 },
        { 1.0, 0.0 },
        { 1.0, 1.0 }
    };

    for(unsigned int y = 0; y < totalHeightDivision/2; ++y) {
        for(unsigned int x = 0; x < totalWidthDivsion/2; ++x) {
            for(int i = 0; i < 6; ++i) {

                qreal xN = (2*x + 2*vertexOffsets[i].x) / static_cast<qreal>(totalWidthDivsion);
                qreal yN = (2*y + 2*vertexOffsets[i].y) / static_cast<qreal>(totalHeightDivision);
                qreal xW = xN * wndWidth;
                qreal yW = yN * wndHeight;

                std::shared_ptr<verticsInfo> objptr (new verticsInfo);
                objptr->wndPos.setX(xW);              objptr->wndPos.setY(yW);
                objptr->normPos.setX(xN);             objptr->normPos.setY(yN);
                objptr->normal.setX(0.0);             objptr->normal.setY(0.0);
                objptr->tex.setX(textureCoords[i].x); objptr->tex.setY(textureCoords[i].y);

                m_list_vertics.push_back(std::move(objptr));
            }
        }
    }

    return !m_list_vertics.empty();

}


// Returns true if x is in range [low..high], else false
static bool inRange(int low, int high, int x)
{
    return ((x-high)*(x-low) <= 0.0);
}


QSGNode * QGrid::updatePaintNode(QSGNode * old_node, UpdatePaintNodeData * update_paint_node_data) {
    Q_UNUSED(update_paint_node_data)

    QSGGeometryNode *node = static_cast<QSGGeometryNode *>(old_node);


    if(m_list_vertics.empty() || !inRange(m_qitem_width-2, m_qitem_width+2, int(width()))
                              || !inRange(m_qitem_height-2, m_qitem_height+2, int(height()))) {

         makeGrid(numberof_horzontaldivision, numberof_verticaldivision, width(), height());
         m_qitem_width = int(width());
         m_qitem_height = int(height());
    }


    QSGGeometry *geometry = nullptr;
    if (!node) {
        node = new QSGGeometryNode;
        geometry = new QSGGeometry(QSGGeometry::defaultAttributes_ColoredPoint2D(), m_list_vertics.size());
        geometry->setDrawingMode(GL_TRIANGLES);
        node->setGeometry(geometry);
        node->setFlag(QSGNode::OwnsGeometry);

        QSGVertexColorMaterial *material = new QSGVertexColorMaterial();
        //material->setColor(QColor(255, 0, 0));
        node->setMaterial(material);
        node->setFlag(QSGNode::OwnsMaterial);
    }
    else {
        node = static_cast<QSGGeometryNode *>(old_node);
        geometry = node->geometry();
        geometry->allocate(m_list_vertics.size());
    }

    QSGGeometry::ColoredPoint2D *vertices = geometry->vertexDataAsColoredPoint2D();

    int j = 0;
    for (j = 0; j < m_list_vertics.size(); j=j+1) {

        vertices[j].set(m_list_vertics[j]->wndPos.x(), m_list_vertics[j]->wndPos.y(), static_cast<uchar>((j%2)*255),
                                                            static_cast<uchar>(0),
                                                            static_cast<uchar>((j%2)*255),
                                                            static_cast<uchar>(255));
    }
    qDebug() << "Size " <<  m_list_vertics.size() << " j " << j;

    node->markDirty(QSGNode::DirtyGeometry);

    return node;
}


